Queue = {
    first = 0,
    last = -1,
    calls = {},
    push = function(self, value)
        self.last = self.last + 1
        self.calls[self.last] = value
    end,
    reset = function(self)
        for  i = 0, self.last, 1 do
            self.calls[i] = nil
            end
            self.first = 0
            self.last = -1;
    end,
    -- Returns element at start of queue and deletes it from queue.
    pop = function(self)
        local first = self.first
        if first > self.last then error("list is empty") end
        local value = self.calls[first]
        self.calls[first] = nil        -- to allow garbage collection
        self.first = first + 1
        return value
  end,

  isEmpty = function(self)
        return self.first > self.last 
    end
  
}
IntervalTaskList = {
    queue = Queue,

    -- Schedule a function onto the Queue
    schedule = function(self ,task, delay)
            -- Make delay an optional parameter. Default to no delay if not specified.
            if(delay == nil) then
                delay = 0
            end
    
            -- Input Validation to ensure task is passed in an is a function. Error out if not.
            if(type(task) ~= "function") then
                osrs.print("ERROR: Must Pass in Function. Passed in type: " .. type(task))
                return
            end
    
            -- Add container associating the function with its delay to the queue so it can be read during execution.
            self.queue:push({task, delay})
    end,
    -- Tell Task List to start running
    startTaskExecution = function(self)
        self:runTaskList()
    end,

    -- Recursive function that iterates through every queued function 
    runTaskList = function(self)
        -- Checks if we've hit end of queue. If so, exit and reset the queue.
        if(self.queue:isEmpty()) then
            osrs.print("Task List Finished")
            self.queue:reset()
            return 
        end

        -- Pop off container with function and delay from front of queue. 
        local currTask = self.queue:pop()
        local callback = currTask[1]
        local delay = currTask[2]

        -- Validate that container elements are of expected type
        if(type(callback) ~= "function" or type(delay) ~= "number") then
            osrs.print("ERROR: Callback is incorrect type. Passed in Callback type: " .. type(callback) .. ", Delay type: " .. type(delay))
            return
        end
        -- Set up async call
        osrs.setInterval(
        -- Create lambda that wraps scheduled callback with recursion call. Delay call by specified seconds.
            function()
                callback()
                self:runTaskList()
            end
        , delay)
    end
}


local platformText = osrs.Ui:inputText(
    {
        label = "platform",
    }
)

local modeText = osrs.Ui:inputText(
    {
        label = "mode",
    }
)

local infoText = osrs.Ui:inputText(
    {
        label = "info",
    }
)

local toolWindow = osrs.Ui:window()

local submit = osrs.Ui:textButton(
    {
        text="Submit",
        width=450,
        onClick = function()
            local pf = platformText.value

            local md = modeText.value

            local ft = infoText.value

            if(pf ~= "win" and pf ~= "mac" and pf ~= "and" and pf ~= "ios") then
                osrs.print("Not a valid platform input - you must use win, mac, ios or and")
                return
            end
            if(md ~= "hdoff" and md ~= "fakesd" and md ~= "defhd" and md ~= "fullhd") then
                osrs.print("Not a valid mode - you must use hdoff, fakesd, defhd or fullhd")
                return
            end    
            osrs.print(platformText.value)
            osrs.print(modeText.value)
            osrs.print(infoText.value)
            runTest(md, pf, ft)
        end
    }
)



--Button to clear side panel
local clear = osrs.Ui:textButton(
    {
        text="Done. Close box.",
    }
)

-- Function to clear panel. Necessary to define after initialization to clear self
clear.onClick = 
    function()
        toolWindow:removeChild(platformText)
        toolWindow:removeChild(modeText)
        toolWindow:removeChild(infoText)
        toolWindow:removeChild(clear)
        osrs.Ui.canvas:removeChild(toolWindow)
    end


toolWindow:addChild(platformText)
toolWindow:addChild(modeText)
toolWindow:addChild(infoText)
toolWindow:addChild(submit)
toolWindow:addChild(clear)

-- Add elements to panel
osrs.Ui.canvas:addChild(toolWindow)

-- Instructions for the text fields
osrs.print("For Platform, enter win, mac, and or ios")
osrs.print("for Mode, enter hdoff, fakesd, defhd or fullhd")
osrs.print("for Info, enter identifier for the test, such as LM or flip-LM")


runTest = function(pf,md,ft)
taskList = IntervalTaskList

--set up char for no renderer benchmarks


osrs.consoleCommand("setstat attack 99")
osrs.consoleCommand("setstat hitpoints 99")
osrs.consoleCommand("setstat defence 99")
osrs.consoleCommand("setstat strength 99")
osrs.consoleCommand("setstat construction 99")
osrs.consoleCommand("setstat ranged 99")
osrs.consoleCommand("setstat magic 99")
osrs.consoleCommand("setstat prayer 99")
osrs.consoleCommand("setstat runecraft 99")
osrs.consoleCommand("setstat mining 99")
osrs.consoleCommand("setstat smithing 99")
osrs.consoleCommand("setstat fishing 99")
osrs.consoleCommand("setstat cooking 99")
osrs.consoleCommand("setstat firemaking 99")
osrs.consoleCommand("setstat woodcutting 99")
osrs.consoleCommand("setstat fishing 99")
osrs.consoleCommand("setstat crafting 99")
osrs.consoleCommand("setstat agility 99")
osrs.consoleCommand("setstat herblore 99")
osrs.consoleCommand("setstat thieving 99")
osrs.consoleCommand("setstat fletching 99")
osrs.consoleCommand("setstat slayer 99")
osrs.consoleCommand("setstat farming 99")
osrs.consoleCommand("setstat hunter 99")
osrs.consoleCommand("setvar spy 4")
osrs.consoleCommand("setvar cookquest 2")
osrs.consoleCommand("setvar corscurs_progress 60")
osrs.consoleCommand("setvar demonslayer_main 3")
osrs.consoleCommand("setvar doricquest 100")
osrs.consoleCommand("setvar dragonquest 10")
osrs.consoleCommand("setvar haunted 3")
osrs.consoleCommand("setvar gobdip_main 6")
osrs.consoleCommand("setvar imp 2")
osrs.consoleCommand("setvar squire 7")
osrs.consoleCommand("setvar mistmyst_progress 135")
osrs.consoleCommand("setvar hunt 4")
osrs.consoleCommand("setvar princequest 110")
osrs.consoleCommand("setvar prieststart 5")
osrs.consoleCommand("setvar rjquest 100")
osrs.consoleCommand("setvar runemysteries 6")
osrs.consoleCommand("setvar sheep 21")
osrs.consoleCommand("setvar phoenixgang 7")
osrs.consoleCommand("setvar blackarmgang 4")
osrs.consoleCommand("setvar vampire 3")
osrs.consoleCommand("setvar hetty 3")
osrs.consoleCommand("setvar cluequest 8")
osrs.consoleCommand("setvar anma_main 240")
osrs.consoleCommand("setvar slice_quest 11")
osrs.consoleCommand("setvar arcquest 14")
osrs.consoleCommand("setvar bcs 108")
osrs.consoleCommand("setvar dwarfrock_quest 110")
osrs.consoleCommand("setvar chompybird 65")
osrs.consoleCommand("setvar biohazard 16")
osrs.consoleCommand("setvar fossilquest_progress 50")
osrs.consoleCommand("setvar fever_quest 140")
osrs.consoleCommand("setvar veos_progress 6")
osrs.consoleCommand("setvar cogquest 8")
osrs.consoleCommand("setvar peng_quest 135")
osrs.consoleCommand("setvar contact 130")
osrs.consoleCommand("setvar fenk_quest 8")
osrs.consoleCommand("setvar myq3_main_quest 320")
osrs.consoleCommand("setvar death_equiproom 80")
osrs.consoleCommand("setvar dttd_main 13")
osrs.consoleCommand("setvar hosidiusquest 11")
osrs.consoleCommand("setvar deserttreasure 15")
osrs.consoleCommand("setvar devious_main 80")
osrs.consoleCommand("setvar ds2 215")
osrs.consoleCommand("setvar itexamlevel 9")
osrs.consoleCommand("setvar dream_prog 28")
osrs.consoleCommand("setvar druidquest 4")
osrs.consoleCommand("setvar mcannon 11")
osrs.consoleCommand("setvar eadgar_quest 110")
osrs.consoleCommand("setvar eaglepeak_quest 40")
osrs.consoleCommand("setvar elemental_workshop_bits 20")
osrs.consoleCommand("setvar elemental_quest_2_main 11")
osrs.consoleCommand("setvar enakh_quest 70")
osrs.consoleCommand("setvar zep_quest 200")
osrs.consoleCommand("setvar eyeglo_quest 60")
osrs.consoleCommand("setvar fairy_farmers_quest 90")
osrs.consoleCommand("setvar fairy2_queencure_quest 90")
osrs.consoleCommand("setvar crestquest 11")
osrs.consoleCommand("setvar feud_var 28")
osrs.consoleCommand("setvar arenaquest 14")
osrs.consoleCommand("setvar fishingcompo 5")
osrs.consoleCommand("setvar forget_quest 140")
osrs.consoleCommand("setvar lovaquest 11")
osrs.consoleCommand("setvar vikingexile 130")
osrs.consoleCommand("setvar fris_quest 340")
osrs.consoleCommand("setvar viking 10")
osrs.consoleCommand("setvar tgod 56")
osrs.consoleCommand("setvar garden_quest 60")
osrs.consoleCommand("setvar fluffs 6")
osrs.consoleCommand("setvar ga 34")
osrs.consoleCommand("setvar ahoy_questvar 8")
osrs.consoleCommand("setvar giantdwarf_quest 50")
osrs.consoleCommand("setvar golem_a 10")
osrs.consoleCommand("setvar grandtree 160")
osrs.consoleCommand("setvar brain_quest_var 130")
osrs.consoleCommand("setvar grim_quest 60")
osrs.consoleCommand("setvar handsand_quest 160")
osrs.consoleCommand("setvar hauntedmine 11")
osrs.consoleCommand("setvar hazeelcultquest 9")
osrs.consoleCommand("setvar heroquest 15")
osrs.consoleCommand("setvar grail 10")
osrs.consoleCommand("setvar horrorquest 10")
osrs.consoleCommand("setvar ics_little_var 26")
osrs.consoleCommand("setvar myreque_2_quest 430")
osrs.consoleCommand("setvar routequest 105")
osrs.consoleCommand("setvar junglepotion 12")
osrs.consoleCommand("setvar kr_quest 90")
osrs.consoleCommand("setvar akd 150	")
osrs.consoleCommand("setvar lotg 56")
osrs.consoleCommand("setvar legendsquest 75")
osrs.consoleCommand("setvar zanaris 6")
osrs.consoleCommand("setvar lost_tribe_quest 11")
osrs.consoleCommand("setvar lunar_quest_main 190")
osrs.consoleCommand("setvar Mastermy2arm_status 200")
osrs.consoleCommand("setvar makinghistory_prog 4")
osrs.consoleCommand("setvar arthur 7")
osrs.consoleCommand("setvar mm_main 9")
osrs.consoleCommand("setvar mm2_progress 200")
osrs.consoleCommand("setvar drunkmonkquest 80")
osrs.consoleCommand("setvar mdaughter_quest_var 70")
osrs.consoleCommand("setvar mourning_quest 9")
osrs.consoleCommand("setvar mourning_quest_main 60")
osrs.consoleCommand("setvar murderquest 2")
osrs.consoleCommand("setvar myarm 320")
osrs.consoleCommand("setvar druidspirit 110")
osrs.consoleCommand("setvar tobquest 86")
osrs.consoleCommand("setvar itgronigen 7")
osrs.consoleCommand("setvar olaf_quest_var 80")
osrs.consoleCommand("setvar onesmallfavour 285")
osrs.consoleCommand("setvar elenaquest 29")
osrs.consoleCommand("setvar porcine 40")
osrs.consoleCommand("setvar priestperil 61")
osrs.consoleCommand("setvar piscquest 13")
osrs.consoleCommand("setvar rag_quest_var 4")
osrs.consoleCommand("setvar rag_quest_var 6")
osrs.consoleCommand("setvar ratcatch_var 127")
osrs.consoleCommand("setvar hundred_main_quest_var 5")
osrs.consoleCommand("setvar rd_main 2")
osrs.consoleCommand("setvar regicide_quest 15")
osrs.consoleCommand("setvar roving_elves_quest 6")
osrs.consoleCommand("setvar royal_quest 30")
osrs.consoleCommand("setvar deal_quest 19")
osrs.consoleCommand("setvar scorpcatcher 6")
osrs.consoleCommand("setvar seaslugquest 12")
osrs.consoleCommand("setvar sotn 90")
osrs.consoleCommand("setvar morttonquest 85")
osrs.consoleCommand("setvar agrith_quest 125")
osrs.consoleCommand("setvar sheepherderquest 3")
osrs.consoleCommand("setvar zombiequeen 15")
osrs.consoleCommand("setvar myq5 138")	
osrs.consoleCommand("setvar sleeping_giants 30")
osrs.consoleCommand("setvar slug2_main 14")
osrs.consoleCommand("setvar sote 200")
osrs.consoleCommand("setvar soulbane_prog 13")
osrs.consoleCommand("setvar elidquest 60")
osrs.consoleCommand("setvar swansong 200")
osrs.consoleCommand("setvar tbwt_main 6")
osrs.consoleCommand("setvar twocats_quest 70")
osrs.consoleCommand("setvar shayzienquest 17")
osrs.consoleCommand("setvar myq4 165")
osrs.consoleCommand("setvar tog_juna_bowl 2")
osrs.consoleCommand("setvar ikov 80")
osrs.consoleCommand("setvar tote 130")
osrs.consoleCommand("setvar misc_quest 100")
osrs.consoleCommand("setvar desertrescue 30")
osrs.consoleCommand("setvar tol_prog 18")
osrs.consoleCommand("setvar treequest 9")
osrs.consoleCommand("setvar totemquest 5")
osrs.consoleCommand("setvar troll_love 45")
osrs.consoleCommand("setvar troll_quest 50")
osrs.consoleCommand("setvar upass 11")
osrs.consoleCommand("setvar wanted_main 11")
osrs.consoleCommand("setvar itwatchtower 13")
osrs.consoleCommand("setvar waterfall_quest 10")
osrs.consoleCommand("setvar surok_quest 150")
osrs.consoleCommand("setvar ballquest 7")
osrs.consoleCommand("setvar zogre 14")
osrs.consoleCommand("setvar zeah_arceuus 1000")
osrs.consoleCommand("setvar zeah_lovakengj 1000")
osrs.consoleCommand("setvar zeah_piscarilius 1000")
osrs.consoleCommand("setvar zeah_hosidius 1000")
osrs.consoleCommand("setvar zeah_shayzien 1000")
osrs.consoleCommand("~runes")

taskList:schedule(function() 
osrs.consoleCommand("setvis 2")
osrs.consoleCommand("~clearinv")
osrs.consoleCommand("~client_classic_resizeable")
--osrs.consoleCommand("give lunar_seal_of_passage")
--osrs.consoleCommand("give lunar_moonclan_liminal_staff")
osrs.consoleCommand("setvar spellbook 2")
osrs.consoleCommand("setvar fairyring_use 1")
osrs.consoleCommand("setvar fairyring_permission 2") 
osrs.print("Character setup complete. Setting render mode.")
end, 8)

taskList:schedule(function()
osrs.consoleCommand("rt " .. pf)
end, 5)

-- Records the Goblin Bridge benchmark

taskList:schedule(function()
    osrs.print("Starting Goblin Bridge benchmark.")
    osrs.consoleCommand("tele 0,50,50,59,30")
    end, 8)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 50")
    end, 5)

taskList:schedule(function()
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)

-- Records the Burthorpe Gameroom benchmark

taskList:schedule(function()
    osrs.print("Goblin Bridge complete. Starting Burthorpe benchmark.")
    osrs.consoleCommand("tele 0,34,77,31,11")
    end, 10)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 50")
    end, 5)

taskList:schedule(function()
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)

-- Records the Arceuus Library benchmark

taskList:schedule(function()
    osrs.print("Burthorpe complete. Starting Arceuus Library benchmark.")
    osrs.consoleCommand("tele 0,25,59,37,33")
    end, 10)
    
taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 50")
    end, 5)
    
taskList:schedule(function()
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)
    
-- Records the God Wars Dungeon benchmark

taskList:schedule(function()
    osrs.print("Arceuus Library complete. Starting God Wars Dungeon benchmark.")
    osrs.consoleCommand("tele 2,45,82,0,49")
    end, 10)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 90")
    end, 5)

taskList:schedule(function()
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)

-- Records the Fossil Island benchmark

taskList:schedule(function()
    osrs.print("God Wars dungeon complete. Starting Fossil Island benchmark.")
    osrs.consoleCommand("tele 0,57,60,35,43")
    end, 10)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 50")
    end, 5)

taskList:schedule(function() 
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)

-- Records the Zenaris benchmark

taskList:schedule(function()
    osrs.print("Fossil Island complete. Starting Zenaris benchmark.")
    osrs.consoleCommand("tele 0,37,69,50,16")
    end, 10)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 50")
    end, 5)

taskList:schedule(function() 
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)

-- Records the Ferox Enclave benchmark

taskList:schedule(function()
    osrs.print("Zenaris complete. Starting Ferox Enclave benchmark.")
    osrs.consoleCommand("tele 0,48,56,62,45")
    end, 10)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 50")
    end, 5)

taskList:schedule(function()
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 5)

-- Records the Draymor with bots benchmark

taskList:schedule(function()
    osrs.print("Ferox Enclave complete. Starting Draymor with bots benchmark.")
    osrs.consoleCommand("tele 0,48,50,9,51")
    end, 10)

taskList:schedule(function()
    osrs.consoleCommand("~client_max_zoom")
    osrs.consoleCommand("~client_look_north")
    osrs.consoleCommand("setdrawdistance 90")
    osrs.consoleCommand("startbots 160")
    end, 7)

taskList:schedule(function()
    osrs.consoleCommand("perfcapture start 5000 ".. md .."_".. pf .."_".. ft .. " 500")
    end, 10)

taskList:schedule(function()
    osrs.print("Draymor Village bot test complete")
    end, 4)

-- complete benchmark performance series   

taskList:schedule(function()
    osrs.print("Benchmark Performance Run for ".. md .."_".. pf .."_".. ft .. " Finished!")
    end, 10)

taskList:startTaskExecution()

end